import moment from 'moment'
import SubscriptionServicesRepository from '../Repositories/SubscriptionServicesRepository.js'
import BaseInfoController from '../Controllers/BaseInfoController.js'
import { log } from './LogController.js'
import { executeQuery } from '../Connection/QueriesExecutor.js'
import { financApiRequest } from './../Requests/RequestModule.js'

const getSubscriptionServices = async function ({ CDFILIAL }) {
    return await SubscriptionServicesRepository.getSubscriptionServices({ CDFILIAL })
}

const getSubscribedServicesByConsumer = async function (consDetails, body) {
    return await SubscriptionServicesRepository.getSubscribedServicesByConsumer({
        CDCLIENTE: consDetails.CDCLIENTE,
        CDFILIAL: consDetails.CDFILIAL,
        CDCONSUMIDOR: body.CDCONSUMIDOR,
        NRCPFRESPCON: consDetails.NRCPFRESPCON
    })
}

const addSubscriptionService = async function (body, consDetails) {
    try {
        const { services, dependent } = body
        const { CDCLIENTE, CDFILIAL, NRCPFRESPCON } = consDetails
        const NRSERVASSCLI = await getNewCode('NCSERVASSCLI', 10)
        const CDCONSUMIDOR = dependent.CDCONSUMIDOR
        let VLRASS = 0
        let DTASSINATURA = new Date()
        
        for (let i = 0; i < services.length; i++) {
            const service = services[i]
            
            let NUMPARCELAS = getNumParcelas(service)
            let VRNCSERVASS = getServicePrice(service, NUMPARCELAS)

            let params1 = { NRSERVASSCLI, NRSERVASS: service.NRSERVASS, VRNCSERVASS, NUMPARCELAS }
            await SubscriptionServicesRepository.insertServicoAssinaturaAssoc(params1)
            VLRASS = Math.round((parseFloat(VLRASS) + parseFloat(VRNCSERVASS)) * 100) / 100
        }

        let params2 = { CDCLIENTE, CDFILIAL, CDCONSUMIDOR, NRCPFRESPCON, NRSERVASSCLI, VLRASS, DTASSINATURA }
        await SubscriptionServicesRepository.insertServicoAssinaturaCliente(params2)

        let apiResponse = await financApiRequest('geraTitulosPorAssinatura', { CDFILIAL, NRCPFRESPCON, NRSERVASSCLI })
        // console.log(apiResponse)

        if (apiResponse?.status === 200 && !apiResponse.data.error && apiResponse.data?.dataset?.geraTitulosPorAssinatura?.success === 'S') {
            return { subscriptionOK: true }
        } else {
            let errorMsg = 'Não foi possível gerar os boletos através da API do Financeiro'
            if (apiResponse?.data?.error) errorMsg += `\nMensagem de Erro da API: ${apiResponse.data.error}`
            throw new Error(errorMsg)
        }
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro no processo de assinatura de serviços:\n${error.message}` })
        return { subscriptionOK: false }
    }   
}

const getNewCode = async (tableName, size) => {
    const baseInfo = await BaseInfoController.getBaseInfo()
    const NRORG = baseInfo[0].NRORG

    const paramsNewCode = { NRORGNOVOCODIGO: NRORG, NOMETABELA: tableName }
    let newCode = await executeQuery({ rawReturn: true, binds: paramsNewCode, newCode: true })
    if (newCode?.length && size) newCode = newCode.substring(newCode.length - size)

    return newCode
}

const getNumParcelas = (service) => {
    let numParcelas = 1
    // A função .diff() do moment.js desconsidera frações de mês. Para respeitar a regra de que assinaturas feitas no
    // início, meio ou fim do mês sempre pagarão o mês integral, é preciso considerar as datas iniciais e finais sempre
    // no primeiro e último dias dos meses. Isso força o moment.js a não desconsiderar as frações
    let firstDayOfCurrentMonth = moment().startOf('month')

    const inicioVigencia = moment(service.DTVINGINI).startOf('month')
    const fimVigencia = moment(service.DTVINGFIM).endOf('month')

    let diffVigenciaTotal = fimVigencia.diff(inicioVigencia, 'months') + 1
    let diffVigenciaAtual = fimVigencia.diff(firstDayOfCurrentMonth, 'months') + 1

    numParcelas = diffVigenciaAtual < diffVigenciaTotal ? diffVigenciaAtual : diffVigenciaTotal

    return numParcelas
}

const getServicePrice = (service, NUMPARCELAS) => {
    const inicioVigencia = moment(service.DTVINGINI).startOf('month')
    const fimVigencia = moment(service.DTVINGFIM).endOf('month')
    let numParcelasTotal = fimVigencia.diff(inicioVigencia, 'months') + 1

    if (NUMPARCELAS === numParcelasTotal) {
        return service.VRNCSERVASS
    } else {
        let priceForMonth = Math.round((parseFloat(service.VRNCSERVASS) / parseFloat(numParcelasTotal)) * 100) / 100
        let servicePrice = Math.round((parseFloat(priceForMonth) * parseFloat(NUMPARCELAS)) * 100) / 100
        return servicePrice
    }
}

const hasAnySubscriptionService = async function ({ CDFILIAL }) {
    const result = await SubscriptionServicesRepository.hasAnySubscriptionService({ CDFILIAL })
    if (result[0] && result[0].SERVICOS && result[0].SERVICOS > 0) {
        return true
    }
    return false
}
export { getSubscriptionServices, addSubscriptionService, getSubscribedServicesByConsumer, hasAnySubscriptionService }
